@@ -3,8 +3,9 @@ package com.fasterxml.jackson.core.json.async;
 import java.io.*;
 
 import com.fasterxml.jackson.core.*;
-import com.fasterxml.jackson.core.base.ParserBase;
+import com.fasterxml.jackson.core.exc.StreamConstraintsException;
 import com.fasterxml.jackson.core.io.IOContext;
+import com.fasterxml.jackson.core.json.JsonParserBase;
 import com.fasterxml.jackson.core.json.JsonReadContext;
 import com.fasterxml.jackson.core.sym.ByteQuadsCanonicalizer;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
@@ -13,9 +14,11 @@ import static com.fasterxml.jackson.core.JsonTokenId.*;
 
 /**
  * Intermediate base class for non-blocking JSON parsers.
+ *
+ * @since 2.9
  */
 public abstract class NonBlockingJsonParserBase
-    extends ParserBase
+    extends JsonParserBase
 {
     /*
     /**********************************************************************
@@ -98,6 +101,8 @@ public abstract class NonBlockingJsonParserBase
 
     protected final static int MINOR_VALUE_TOKEN_NON_STD = 19;
 
+    protected final static int MINOR_NUMBER_PLUS = 22;
+
     protected final static int MINOR_NUMBER_MINUS = 23;
     protected final static int MINOR_NUMBER_ZERO = 24; // zero as first, possibly trimming multiple
     protected final static int MINOR_NUMBER_MINUSZERO = 25; // "-0" (and possibly more zeroes) receive
@@ -128,7 +133,7 @@ public abstract class NonBlockingJsonParserBase
     protected final static int MINOR_COMMENT_C = 53;
     protected final static int MINOR_COMMENT_CPP = 54;
     protected final static int MINOR_COMMENT_YAML = 55;
-    
+
     /*
     /**********************************************************************
     /* Helper objects, symbols (field names)
@@ -211,7 +216,7 @@ public abstract class NonBlockingJsonParserBase
             Double.NaN,
             Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY,
     };
-    
+
     /**
      * When tokenizing non-standard ("odd") tokens, this is the type to consider;
      * also works as index to actual textual representation.
@@ -230,7 +235,7 @@ public abstract class NonBlockingJsonParserBase
      * logical offset within input "stream"
      */
     protected int _currBufferStart = 0;
-    
+
     /**
      * Alternate row tracker, used to keep track of position by `\r` marker
      * (whereas <code>_currInputRow</code> tracks `\n`). Used to simplify
@@ -239,7 +244,7 @@ public abstract class NonBlockingJsonParserBase
      * case we can simply choose max of two row candidates.
      */
     protected int _currInputRowAlt = 1;
-    
+
     /*
     /**********************************************************************
     /* Life-cycle
@@ -249,27 +254,25 @@ public abstract class NonBlockingJsonParserBase
     public NonBlockingJsonParserBase(IOContext ctxt, int parserFeatures,
             ByteQuadsCanonicalizer sym)
     {
-        super(ctxt, parserFeatures);
+        super(ctxt, parserFeatures, null);
         _symbols = sym;
         _currToken = null;
         _majorState = MAJOR_INITIAL;
         _majorStateAfterValue = MAJOR_ROOT;
     }
 
-    @Override
-    public ObjectCodec getCodec() {
-        return null;
-    }
-
     @Override
     public void setCodec(ObjectCodec c) {
         throw new UnsupportedOperationException("Can not use ObjectMapper with non-blocking parser");
     }
 
-    /**
-     * @since 2.9
+    /*
+    /**********************************************************
+    /* Capability introspection
+    /**********************************************************
      */
-    @Override
+    
+    @Override // since 2.9
     public boolean canParseAsync() { return true; }
 
     /*
@@ -337,20 +340,30 @@ public abstract class NonBlockingJsonParserBase
     }
 
     @Override
-    public JsonLocation getCurrentLocation()
+    public JsonLocation currentLocation()
     {
         int col = _inputPtr - _currInputRowStart + 1; // 1-based
         // Since we track CR and LF separately, max should gives us right answer
         int row = Math.max(_currInputRow, _currInputRowAlt);
-        return new JsonLocation(_getSourceReference(),
+        return new JsonLocation(_contentReference(),
                 _currInputProcessed + (_inputPtr - _currBufferStart), -1L, // bytes, chars
                 row, col);
     }
 
+    @Override // @since 2.17
+    protected JsonLocation _currentLocationMinusOne() {
+        final int prevInputPtr = _inputPtr - 1;
+        int row = Math.max(_currInputRow, _currInputRowAlt);
+        final int col = prevInputPtr - _currInputRowStart + 1; // 1-based
+        return new JsonLocation(_contentReference(),
+                _currInputProcessed + (prevInputPtr - _currBufferStart), -1L, // bytes, chars
+                row, col);
+    }
+
     @Override
-    public JsonLocation getTokenLocation()
+    public JsonLocation currentTokenLocation()
     {
-        return new JsonLocation(_getSourceReference(),
+        return new JsonLocation(_contentReference(),
                 _tokenInputTotal, -1L, _tokenInputRow, _tokenInputCol);
     }
 
@@ -365,6 +378,9 @@ public abstract class NonBlockingJsonParserBase
      * if no current event (before first call to {@link #nextToken}, or
      * after encountering end-of-input), returns null.
      * Method can be called for any event.
+     *
+     * @throws IOException if there are general I/O or parse issues, including if the text is too large,
+     * see {@link com.fasterxml.jackson.core.StreamReadConstraints.Builder#maxStringLength(int)}
      */
     @Override
     public String getText() throws IOException
@@ -375,7 +391,7 @@ public abstract class NonBlockingJsonParserBase
         return _getText2(_currToken);
     }
 
-    protected final String _getText2(JsonToken t)
+    protected final String _getText2(JsonToken t) throws IOException
     {
         if (t == null) {
             return null;
@@ -422,7 +438,7 @@ public abstract class NonBlockingJsonParserBase
     }
 
     // // // Let's override default impls for improved performance
-    
+
     // @since 2.1
     @Override
     public String getValueAsString() throws IOException
@@ -431,11 +447,11 @@ public abstract class NonBlockingJsonParserBase
             return _textBuffer.contentsAsString();
         }
         if (_currToken == JsonToken.FIELD_NAME) {
-            return getCurrentName();
+            return currentName();
         }
         return super.getValueAsString(null);
     }
-    
+
     // @since 2.1
     @Override
     public String getValueAsString(String defValue) throws IOException
@@ -444,7 +460,7 @@ public abstract class NonBlockingJsonParserBase
             return _textBuffer.contentsAsString();
         }
         if (_currToken == JsonToken.FIELD_NAME) {
-            return getCurrentName();
+            return currentName();
         }
         return super.getValueAsString(defValue);
     }
@@ -454,7 +470,7 @@ public abstract class NonBlockingJsonParserBase
     {
         if (_currToken != null) { // null only before/after document
             switch (_currToken.id()) {
-                
+
             case ID_FIELD_NAME:
                 if (!_nameCopied) {
                     String name = _parsingContext.getCurrentName();
@@ -468,13 +484,13 @@ public abstract class NonBlockingJsonParserBase
                     _nameCopied = true;
                 }
                 return _nameCopyBuffer;
-    
+
             case ID_STRING:
                 // fall through
             case ID_NUMBER_INT:
             case ID_NUMBER_FLOAT:
                 return _textBuffer.getTextBuffer();
-                
+
             default:
                 return _currToken.asCharArray();
             }
@@ -487,7 +503,7 @@ public abstract class NonBlockingJsonParserBase
     {
         if (_currToken != null) { // null only before/after document
             switch (_currToken.id()) {
-                
+
             case ID_FIELD_NAME:
                 return _parsingContext.getCurrentName().length();
             case ID_STRING:
@@ -495,7 +511,7 @@ public abstract class NonBlockingJsonParserBase
             case ID_NUMBER_INT:
             case ID_NUMBER_FLOAT:
                 return _textBuffer.size();
-                
+
             default:
                 return _currToken.asCharArray().length;
             }
@@ -569,7 +585,7 @@ public abstract class NonBlockingJsonParserBase
 
     protected final JsonToken _startArrayScope() throws IOException
     {
-        _parsingContext = _parsingContext.createChildArrayContext(-1, -1);
+        createChildArrayContext(-1, -1);
         _majorState = MAJOR_ARRAY_ELEMENT_FIRST;
         _majorStateAfterValue = MAJOR_ARRAY_ELEMENT_NEXT;
         return (_currToken = JsonToken.START_ARRAY);
@@ -577,7 +593,7 @@ public abstract class NonBlockingJsonParserBase
 
     protected final JsonToken _startObjectScope() throws IOException
     {
-        _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
+        createChildObjectContext(-1, -1);
         _majorState = MAJOR_OBJECT_FIELD_FIRST;
         _majorStateAfterValue = MAJOR_OBJECT_FIELD_NEXT;
         return (_currToken = JsonToken.START_OBJECT);
@@ -629,7 +645,8 @@ public abstract class NonBlockingJsonParserBase
     /**********************************************************
      */
 
-    protected final String _findName(int q1, int lastQuadBytes) throws JsonParseException
+    protected final String _findName(int q1, int lastQuadBytes)
+            throws JsonParseException, StreamConstraintsException
     {
         q1 = _padLastQuad(q1, lastQuadBytes);
         // Usually we'll find it from the canonical symbol table already
@@ -642,7 +659,8 @@ public abstract class NonBlockingJsonParserBase
         return _addName(_quadBuffer, 1, lastQuadBytes);
     }
 
-    protected final String _findName(int q1, int q2, int lastQuadBytes) throws JsonParseException
+    protected final String _findName(int q1, int q2, int lastQuadBytes)
+            throws JsonParseException, StreamConstraintsException
     {
         q2 = _padLastQuad(q2, lastQuadBytes);
         // Usually we'll find it from the canonical symbol table already
@@ -656,7 +674,8 @@ public abstract class NonBlockingJsonParserBase
         return _addName(_quadBuffer, 2, lastQuadBytes);
     }
 
-    protected final String _findName(int q1, int q2, int q3, int lastQuadBytes) throws JsonParseException
+    protected final String _findName(int q1, int q2, int q3, int lastQuadBytes)
+            throws JsonParseException, StreamConstraintsException
     {
         q3 = _padLastQuad(q3, lastQuadBytes);
         String name = _symbols.findName(q1, q2, q3);
@@ -670,20 +689,20 @@ public abstract class NonBlockingJsonParserBase
         return _addName(quads, 3, lastQuadBytes);
     }
 
-    /**
-     * This is the main workhorse method used when we take a symbol
-     * table miss. It needs to demultiplex individual bytes, decode
-     * multi-byte chars (if any), and then construct Name instance
-     * and add it to the symbol table.
-     */
-    protected final String _addName(int[] quads, int qlen, int lastQuadBytes) throws JsonParseException
+    // This is the main workhorse method used when we take a symbol
+    // table miss. It needs to demultiplex individual bytes, decode
+    // multi-byte chars (if any), and then construct Name instance
+    // and add it to the symbol table.
+    protected final String _addName(int[] quads, int qlen, int lastQuadBytes)
+            throws JsonParseException, StreamConstraintsException
     {
         /* Ok: must decode UTF-8 chars. No other validation is
-         * needed, since unescaping has been done earlier as necessary
+         * needed, since unescaping has been done earlier, as necessary
          * (as well as error reporting for unescaped control chars)
          */
         // 4 bytes per quad, except last one maybe less
-        int byteLen = (qlen << 2) - 4 + lastQuadBytes;
+        final int byteLen = (qlen << 2) - 4 + lastQuadBytes;
+        _streamReadConstraints.validateNameLength(byteLen);
 
         /* And last one is not correctly aligned (leading zero bytes instead
          * need to shift a bit, instead of trailing). Only need to shift it
@@ -728,13 +747,13 @@ public abstract class NonBlockingJsonParserBase
                 if ((ix + needed) > byteLen) {
                     _reportInvalidEOF(" in field name", JsonToken.FIELD_NAME);
                 }
-                
+
                 // Ok, always need at least one more:
                 int ch2 = quads[ix >> 2]; // current quad, need to shift+mask
                 byteIx = (ix & 3);
                 ch2 = (ch2 >> ((3 - byteIx) << 3));
                 ++ix;
-                
+
                 if ((ch2 & 0xC0) != 0x080) {
                     _reportInvalidOther(ch2);
                 }
@@ -744,7 +763,7 @@ public abstract class NonBlockingJsonParserBase
                     byteIx = (ix & 3);
                     ch2 = (ch2 >> ((3 - byteIx) << 3));
                     ++ix;
-                    
+
                     if ((ch2 & 0xC0) != 0x080) {
                         _reportInvalidOther(ch2);
                     }
@@ -777,6 +796,10 @@ public abstract class NonBlockingJsonParserBase
 
         // Ok. Now we have the character array, and can construct the String
         String baseName = new String(cbuf, 0, cix);
+        // 5-May-2023, ckozak: [core#1015] respect CANONICALIZE_FIELD_NAMES factory config.
+        if (!_symbols.isCanonicalizing()) {
+            return baseName;
+        }
         // And finally, un-align if necessary
         if (lastQuadBytes < 4) {
             quads[qlen-1] = lastQuad;
@@ -784,9 +807,7 @@ public abstract class NonBlockingJsonParserBase
         return _symbols.addName(baseName, quads, qlen);
     }
 
-    /**
-     * Helper method needed to fix [jackson-core#148], masking of 0x00 character
-     */
+    // Helper method needed to fix [jackson-core#148], masking of 0x00 character
     protected final static int _padLastQuad(int q, int bytes) {
         return (bytes == 4) ? q : (q | (-1 << (bytes << 3)));
     }
@@ -796,11 +817,9 @@ public abstract class NonBlockingJsonParserBase
     /* Internal methods, state changes
     /**********************************************************************
      */
-    
-    /**
-     * Helper method called at point when all input has been exhausted and
-     * input feeder has indicated no more input will be forthcoming.
-     */
+
+    // Helper method called at point when all input has been exhausted and
+    // input feeder has indicated no more input will be forthcoming.
     protected final JsonToken _eofAsNextToken() throws IOException {
         _majorState = MAJOR_CLOSED;
         if (!_parsingContext.inRoot()) {
@@ -842,7 +861,7 @@ public abstract class NonBlockingJsonParserBase
         String tokenStr = NON_STD_TOKENS[type];
         _textBuffer.resetWithString(tokenStr);
         if (!isEnabled(Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
-            _reportError("Non-standard token '%s': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow",
+            _reportError("Non-standard token '%s': enable `JsonReadFeature.ALLOW_NON_NUMERIC_NUMBERS` to allow",
                     tokenStr);
         }
         _intLength = 0;
