@@ -20,4 +20,8 @@ module com.fasterxml.jackson.core {
 
     // 03-Oct-2019, tatu: [core#567] Add self-use to avoid warnings
     uses com.fasterxml.jackson.core.ObjectCodec;
+
+    // 25-Sep-2024: [core#1340] Need to explicitly add even in base
+    provides com.fasterxml.jackson.core.JsonFactory with
+        com.fasterxml.jackson.core.JsonFactory;
 }
