@@ -3,23 +3,34 @@ package com.fasterxml.jackson.core.io;
 import java.io.StringWriter;
 import java.util.Random;
 
-import static org.junit.Assert.*;
+import org.junit.jupiter.api.Test;
+
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 
 import com.fasterxml.jackson.core.*;
 
-public class TestJsonStringEncoder
-    extends com.fasterxml.jackson.core.BaseTest
+class TestJsonStringEncoder
+        extends com.fasterxml.jackson.core.JUnit5TestBase
 {
-    public void testQuoteAsString() throws Exception
+    @Test
+    void quoteAsString() throws Exception
     {
         JsonStringEncoder encoder = new JsonStringEncoder();
         char[] result = encoder.quoteAsString("foobar");
         assertArrayEquals("foobar".toCharArray(), result);
         result = encoder.quoteAsString("\"x\"");
         assertArrayEquals("\\\"x\\\"".toCharArray(), result);
+
+        // and simply for sake of code coverage
+        result = encoder.quoteAsString(new StringBuilder("foobar"));
+        assertArrayEquals("foobar".toCharArray(), result);
+        result = encoder.quoteAsString(new StringBuilder("\"x\""));
+        assertArrayEquals("\\\"x\\\"".toCharArray(), result);
     }
 
-    public void testQuoteCharSequenceAsString() throws Exception
+    @Test
+    void quoteCharSequenceAsString() throws Exception
     {
         StringBuilder output = new StringBuilder();
         StringBuilder builder = new StringBuilder();
@@ -34,7 +45,8 @@ public class TestJsonStringEncoder
     }
 
     // For [JACKSON-853]
-    public void testQuoteLongAsString() throws Exception
+    @Test
+    void quoteLongAsString() throws Exception
     {
         JsonStringEncoder encoder = new JsonStringEncoder();
         StringBuilder sb = new StringBuilder();
@@ -48,10 +60,10 @@ public class TestJsonStringEncoder
         char[] result = encoder.quoteAsString(input);
         assertEquals(2*input.length(), result.length);
         assertEquals(exp, new String(result));
-        
     }
 
-    public void testQuoteLongCharSequenceAsString() throws Exception
+    @Test
+    void quoteLongCharSequenceAsString() throws Exception
     {
         StringBuilder output = new StringBuilder();
         StringBuilder input = new StringBuilder();
@@ -67,7 +79,8 @@ public class TestJsonStringEncoder
 
     }
 
-    public void testQuoteAsUTF8() throws Exception
+    @Test
+    void quoteAsUTF8() throws Exception
     {
         // In this case, let's actually use existing JsonGenerator to produce expected values
         JsonFactory f = new JsonFactory();
@@ -90,7 +103,8 @@ public class TestJsonStringEncoder
         }
     }
 
-    public void testEncodeAsUTF8() throws Exception
+    @Test
+    void encodeAsUTF8() throws Exception
     {
         JsonStringEncoder encoder = new JsonStringEncoder();
         String[] strings = new String[] {
@@ -100,11 +114,15 @@ public class TestJsonStringEncoder
                 generateRandom(39000)
         };
         for (String str : strings) {
-            assertArrayEquals(str.getBytes("UTF-8"), encoder.encodeAsUTF8(str));
+            final byte[] exp = str.getBytes("UTF-8");
+            assertArrayEquals(exp, encoder.encodeAsUTF8(str));
+            // and for 2.x code coverage (only
+            assertArrayEquals(exp, encoder.encodeAsUTF8((CharSequence)str));
         }
     }
 
-    public void testCtrlChars() throws Exception
+    @Test
+    void ctrlChars() throws Exception
     {
         char[] input = new char[] { 0, 1, 2, 3, 4 };
         char[] quoted = JsonStringEncoder.getInstance().quoteAsString(new String(input));
@@ -112,7 +130,8 @@ public class TestJsonStringEncoder
     }
 
     // [JACKSON-884]
-    public void testCharSequenceWithCtrlChars() throws Exception
+    @Test
+    void charSequenceWithCtrlChars() throws Exception
     {
         char[] input = new char[] { 0, 1, 2, 3, 4 };
         StringBuilder builder = new StringBuilder();
@@ -122,13 +141,57 @@ public class TestJsonStringEncoder
         assertEquals("\\u0000\\u0001\\u0002\\u0003\\u0004", output.toString());
     }
 
+    // [core#712]: simple sanity checks for calculation logic
+    @Test
+    void byteBufferDefaultSize()
+    {
+        // byte size is simple, x2 except below buffer size 24
+        assertEquals(JsonStringEncoder.MIN_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(1));
+        assertEquals(JsonStringEncoder.MIN_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(11));
+
+        assertEquals(36, JsonStringEncoder._initialByteBufSize(20));
+        assertEquals(73, JsonStringEncoder._initialByteBufSize(45));
+        assertEquals(1506, JsonStringEncoder._initialByteBufSize(1000));
+        assertEquals(9006, JsonStringEncoder._initialByteBufSize(6000));
+
+        // and up to max initial size
+        assertEquals(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE + 1));
+        assertEquals(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(999999));
+    }
+
+    // [core#712]: simple sanity checks for calculation logic
+    @Test
+    void charBufferDefaultSize()
+    {
+        // char[] bit more complex, starts with minimum size of 16
+        assertEquals(JsonStringEncoder.MIN_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(1));
+        assertEquals(JsonStringEncoder.MIN_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(8));
+
+        // and then grows by ~5%
+        assertEquals(62, JsonStringEncoder._initialCharBufSize(50));
+        assertEquals(118, JsonStringEncoder._initialCharBufSize(100));
+        assertEquals(1131, JsonStringEncoder._initialCharBufSize(1000));
+        assertEquals(9000, JsonStringEncoder._initialCharBufSize(8000));
+
+        // up to max, simi
+        assertEquals(JsonStringEncoder.MAX_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(32000));
+        assertEquals(JsonStringEncoder.MAX_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(900000));
+    }
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Helper methods
-    /**********************************************************
+    /**********************************************************************
      */
 
-    
     private String generateRandom(int length)
     {
         StringBuilder sb = new StringBuilder(length);
