@@ -3,21 +3,25 @@ package com.fasterxml.jackson.core.read;
 import java.io.*;
 
 import com.fasterxml.jackson.core.*;
+
+import org.junit.jupiter.api.Test;
 import com.fasterxml.jackson.core.json.JsonReadFeature;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 /**
  * Unit tests for verifying that support for (non-standard) comments
  * works as expected.
  */
-public class CommentParsingTest
-    extends com.fasterxml.jackson.core.BaseTest
+class CommentParsingTest
+        extends JUnit5TestBase
 {
     final static String DOC_WITH_SLASHSTAR_COMMENT =
-        "[ /* comment:\n ends here */ 1 /* one more ok to have \"unquoted\"  */ ]"
+        "[ /* comment:\n ends here */ 1 /* one more ok to have \"unquoted\" and non-ascii: \u3456 \u00A0  */ ]"
         ;
 
     final static String DOC_WITH_SLASHSLASH_COMMENT =
-        "[ // comment...\n 1 \r  // one more, not array: []   \n ]"
+        "[ // comment...\n 1 \r  // one more, not array: [] \u00A0 & \u3456  \n ]"
         ;
 
     /*
@@ -25,12 +29,13 @@ public class CommentParsingTest
     /* Test method wrappers
     /**********************************************************
      */
-    
+
     /**
      * Unit test for verifying that by default comments are not
      * recognized.
      */
-    public void testDefaultSettings() throws Exception
+    @Test
+    void defaultSettings() throws Exception
     {
         JsonFactory jf = new JsonFactory();
         assertFalse(jf.isEnabled(JsonParser.Feature.ALLOW_COMMENTS));
@@ -39,7 +44,8 @@ public class CommentParsingTest
         p.close();
     }
 
-    public void testCommentsDisabled() throws Exception
+    @Test
+    void commentsDisabled() throws Exception
     {
         _testDisabled(DOC_WITH_SLASHSTAR_COMMENT, MODE_INPUT_STREAM);
         _testDisabled(DOC_WITH_SLASHSLASH_COMMENT, MODE_INPUT_STREAM);
@@ -51,7 +57,8 @@ public class CommentParsingTest
         _testDisabled(DOC_WITH_SLASHSLASH_COMMENT, MODE_DATA_INPUT);
     }
 
-    public void testCommentsEnabled() throws Exception
+    @Test
+    void commentsEnabled() throws Exception
     {
         _testEnabled(DOC_WITH_SLASHSTAR_COMMENT, MODE_INPUT_STREAM);
         _testEnabled(DOC_WITH_SLASHSLASH_COMMENT, MODE_INPUT_STREAM);
@@ -63,7 +70,8 @@ public class CommentParsingTest
         _testEnabled(DOC_WITH_SLASHSLASH_COMMENT, MODE_DATA_INPUT);
     }
 
-    public void testCommentsWithUTF8() throws Exception
+    @Test
+    void commentsWithUTF8() throws Exception
     {
         final String JSON = "/* \u00a9 2099 Yoyodyne Inc. */\n [ \"bar? \u00a9\" ]\n";
         _testWithUTF8Chars(JSON, MODE_INPUT_STREAM);
@@ -72,7 +80,8 @@ public class CommentParsingTest
         _testWithUTF8Chars(JSON, MODE_DATA_INPUT);
     }
 
-    public void testYAMLCommentsBytes() throws Exception {
+    @Test
+    void yamlCommentsBytes() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_YAML_COMMENTS)
                 .build();
@@ -85,7 +94,8 @@ public class CommentParsingTest
         _testCommentsBeforePropValue(f, MODE_DATA_INPUT, "# foo\n");
     }
 
-    public void testYAMLCommentsChars() throws Exception {
+    @Test
+    void yamlCommentsChars() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_YAML_COMMENTS)
                 .build();
@@ -95,7 +105,8 @@ public class CommentParsingTest
         _testCommentsBetweenArrayValues(f, MODE_READER, COMMENT);
     }
 
-    public void testCCommentsBytes() throws Exception {
+    @Test
+    void cCommentsBytes() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_JAVA_COMMENTS)
                 .build();
@@ -105,7 +116,8 @@ public class CommentParsingTest
         _testCommentsBeforePropValue(f, MODE_DATA_INPUT, COMMENT);
     }
 
-    public void testCCommentsChars() throws Exception {
+    @Test
+    void cCommentsChars() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_JAVA_COMMENTS)
                 .build();
@@ -113,7 +125,8 @@ public class CommentParsingTest
         _testCommentsBeforePropValue(f, MODE_READER, COMMENT);
     }
 
-    public void testCppCommentsBytes() throws Exception {
+    @Test
+    void cppCommentsBytes() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_JAVA_COMMENTS)
                 .build();
@@ -123,7 +136,8 @@ public class CommentParsingTest
         _testCommentsBeforePropValue(f, MODE_DATA_INPUT, COMMENT);
     }
 
-    public void testCppCommentsChars() throws Exception {
+    @Test
+    void cppCommentsChars() throws Exception {
         final JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_JAVA_COMMENTS)
                 .build();
@@ -143,7 +157,7 @@ public class CommentParsingTest
                 ":\t%s123",
         }) {
             String commented = String.format(arg, comment);
-            
+
             final String DOC = "{\"abc\"" + commented + "}";
             JsonParser p = createParser(f, mode, DOC);
             assertEquals(JsonToken.START_OBJECT, p.nextToken());
@@ -165,7 +179,7 @@ public class CommentParsingTest
             assertEquals(JsonToken.END_OBJECT, p.nextToken());
             p.close();
         }
-        
+
     }
 
     @SuppressWarnings("resource")
@@ -184,7 +198,7 @@ public class CommentParsingTest
                 "%s\n,",
         }) {
             String commented = String.format(tmpl, comment);
-            
+
             final String DOC = "[1"+commented+"2]";
             JsonParser p = createParser(f, mode, DOC);
             assertEquals(JsonToken.START_ARRAY, p.nextToken());
@@ -207,9 +221,9 @@ public class CommentParsingTest
             assertEquals(JsonToken.END_ARRAY, p.nextToken());
             p.close();
         }
-        
+
     }
-    
+
     private void _testYAMLComments(JsonFactory f, int mode) throws Exception
     {
         final String DOC = "# foo\n"
@@ -225,11 +239,11 @@ public class CommentParsingTest
         JsonParser p = createParser(f, mode, DOC);
         assertEquals(JsonToken.START_OBJECT, p.nextToken());
         assertEquals(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("a", p.getCurrentName());
+        assertEquals("a", p.currentName());
         assertEquals(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(1, p.getIntValue());
         assertEquals(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("b", p.getCurrentName());
+        assertEquals("b", p.currentName());
         assertEquals(JsonToken.START_ARRAY, p.nextToken());
         assertEquals(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(3, p.getIntValue());
@@ -258,7 +272,7 @@ public class CommentParsingTest
         }
         p.close();
     }
-    
+
     private void _testDisabled(String doc, int mode) throws IOException
     {
         JsonParser p = _createParser(doc, mode, false);
