@@ -8,19 +8,25 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 
 import com.fasterxml.jackson.core.*;
+
+import org.junit.jupiter.api.Test;
 import com.fasterxml.jackson.core.exc.InputCoercionException;
 import com.fasterxml.jackson.core.io.NumberInput;
 import com.fasterxml.jackson.core.json.JsonReadFeature;
 
+import static org.junit.jupiter.api.Assertions.*;
+
 /**
  * Set of basic unit tests for verifying that the basic parser
  * functionality works as expected.
  */
 @SuppressWarnings("resource")
-public class NumberParsingTest
-    extends com.fasterxml.jackson.core.BaseTest
+class NumberParsingTest
+        extends JUnit5TestBase
 {
-    private final JsonFactory FACTORY = sharedStreamFactory();
+    protected JsonFactory jsonFactory() {
+        return sharedStreamFactory();
+    }
 
     /*
     /**********************************************************************
@@ -28,7 +34,8 @@ public class NumberParsingTest
     /**********************************************************************
      */
 
-    public void testSimpleBoolean() throws Exception
+    @Test
+    void simpleBoolean() throws Exception
     {
         _testSimpleBoolean(MODE_INPUT_STREAM);
         _testSimpleBoolean(MODE_INPUT_STREAM_THROTTLED);
@@ -38,10 +45,10 @@ public class NumberParsingTest
 
     private void _testSimpleBoolean(int mode) throws Exception
     {
-        JsonParser p = createParser(mode, "[ true ]");
+        JsonParser p = createParser(jsonFactory(), mode, "[ true ]");
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_TRUE, p.nextToken());
-        assertEquals(true, p.getBooleanValue());
+        assertTrue(p.getBooleanValue());
         assertToken(JsonToken.END_ARRAY, p.nextToken());
         p.close();
     }
@@ -52,7 +59,8 @@ public class NumberParsingTest
     /**********************************************************************
      */
 
-    public void testSimpleInt() throws Exception
+    @Test
+    void simpleInt() throws Exception
     {
         for (int EXP_I : new int[] { 1234, -999, 0, 1, -2, 123456789 }) {
             _testSimpleInt(EXP_I, MODE_INPUT_STREAM);
@@ -65,10 +73,11 @@ public class NumberParsingTest
     private void _testSimpleInt(int EXP_I, int mode) throws Exception
     {
         String DOC = "[ "+EXP_I+" ]";
-        JsonParser p = createParser(mode, DOC);
+        JsonParser p = createParser(jsonFactory(), mode, DOC);
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(JsonParser.NumberType.INT, p.getNumberType());
+        assertEquals(JsonParser.NumberTypeFP.UNKNOWN, p.getNumberTypeFP());
         assertTrue(p.isExpectedNumberIntToken());
         assertEquals(""+EXP_I, p.getText());
 
@@ -100,14 +109,14 @@ public class NumberParsingTest
         assertEquals(EXP_I, p.getIntValue());
         assertEquals(EXP_I, p.getValueAsInt(EXP_I + 3));
         assertEquals(EXP_I, p.getValueAsInt());
-        assertEquals((long) EXP_I, p.getLongValue());
+        assertEquals(EXP_I, p.getLongValue());
         assertEquals((double) EXP_I, p.getDoubleValue());
-        assertEquals(BigDecimal.valueOf((long) EXP_I), p.getDecimalValue());
+        assertEquals(BigDecimal.valueOf(EXP_I), p.getDecimalValue());
         assertToken(JsonToken.END_ARRAY, p.nextToken());
         p.close();
 
         DOC = String.valueOf(EXP_I);
-        p = createParser(mode, DOC + " "); // DataInput requires separator
+        p = createParser(jsonFactory(), mode, DOC + " "); // DataInput requires separator
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertTrue(p.isExpectedNumberIntToken());
         assertEquals(DOC, p.getText());
@@ -115,36 +124,37 @@ public class NumberParsingTest
         int i = p.getIntValue();
 
         assertEquals(EXP_I, i);
-        assertEquals((long) EXP_I, p.getLongValue());
+        assertEquals(EXP_I, p.getLongValue());
         assertEquals((double) EXP_I, p.getDoubleValue());
-        assertEquals(BigDecimal.valueOf((long) EXP_I), p.getDecimalValue());
+        assertEquals(BigDecimal.valueOf(EXP_I), p.getDecimalValue());
         p.close();
 
         // and finally, coercion from double to int; couple of variants
         DOC = String.valueOf(EXP_I)+".0";
-        p = createParser(mode, DOC + " ");
+        p = createParser(jsonFactory(), mode, DOC + " ");
         assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
         assertFalse(p.isExpectedNumberIntToken());
         assertEquals(EXP_I, p.getValueAsInt());
         p.close();
 
-        p = createParser(mode, DOC + " ");
+        p = createParser(jsonFactory(), mode, DOC + " ");
         assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
         assertEquals(EXP_I, p.getValueAsInt(0));
         p.close();
     }
 
-    public void testIntRange() throws Exception
+    @Test
+    void intRange() throws Exception
     {
         // let's test with readers and streams, separate code paths:
         for (int mode : ALL_MODES) {
             String DOC = "[ "+Integer.MAX_VALUE+","+Integer.MIN_VALUE+" ]";
-            JsonParser p = createParser(mode, DOC);
+            JsonParser p = createParser(jsonFactory(), mode, DOC);
             assertToken(JsonToken.START_ARRAY, p.nextToken());
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.INT, p.getNumberType());
             assertEquals(Integer.MAX_VALUE, p.getIntValue());
-    
+
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.INT, p.getNumberType());
             assertEquals(Integer.MIN_VALUE, p.getIntValue());
@@ -152,7 +162,8 @@ public class NumberParsingTest
         }
     }
 
-    public void testIntParsing() throws Exception
+    @Test
+    void intParsing() throws Exception
     {
         char[] testChars = "123456789".toCharArray();
 
@@ -174,7 +185,8 @@ public class NumberParsingTest
         assertEquals(0, NumberInput.parseInt(testChars, 1, 1));
     }
 
-    public void testIntParsingWithStrings() throws Exception
+    @Test
+    void intParsingWithStrings() throws Exception
     {
         assertEquals(3, NumberInput.parseInt("3"));
         assertEquals(3, NumberInput.parseInt("+3"));
@@ -190,7 +202,8 @@ public class NumberParsingTest
         assertEquals(Integer.MAX_VALUE, NumberInput.parseInt(""+Integer.MAX_VALUE));
     }
 
-    public void testLongParsingWithStrings() throws Exception
+    @Test
+    void longParsingWithStrings() throws Exception
     {
         assertEquals(3, NumberInput.parseLong("3"));
         assertEquals(3, NumberInput.parseLong("+3"));
@@ -202,19 +215,83 @@ public class NumberParsingTest
         assertEquals(-131, NumberInput.parseLong("-131"));
         assertEquals(2709, NumberInput.parseLong("2709"));
         assertEquals(-9999, NumberInput.parseLong("-9999"));
+        assertEquals(1234567890123456789L, NumberInput.parseLong("1234567890123456789"));
+        assertEquals(-1234567890123456789L, NumberInput.parseLong("-1234567890123456789"));
         assertEquals(Long.MIN_VALUE, NumberInput.parseLong(""+Long.MIN_VALUE));
         assertEquals(Integer.MIN_VALUE-1, NumberInput.parseLong(""+(Integer.MIN_VALUE-1)));
         assertEquals(Long.MAX_VALUE, NumberInput.parseLong(""+Long.MAX_VALUE));
         assertEquals(Integer.MAX_VALUE+1, NumberInput.parseLong(""+(Integer.MAX_VALUE+1)));
     }
 
+    @Test
+    void intOverflow() {
+        try {
+            // Integer.MAX_VALUE + 1
+            NumberInput.parseInt("2147483648");
+            fail("expected NumberFormatException");
+        } catch (NumberFormatException nfe) {
+            verifyException(nfe, "For input string: \"2147483648\"");
+        }
+        try {
+            // Integer.MIN_VALUE - 1
+            NumberInput.parseInt("-2147483649");
+            fail("expected NumberFormatException");
+        } catch (NumberFormatException nfe) {
+            verifyException(nfe, "For input string: \"-2147483649\"");
+        }
+    }
+
+    @Test
+    void longOverflow() {
+        try {
+            // Long.MAX_VALUE + 1
+            NumberInput.parseLong("9223372036854775808");
+            fail("expected NumberFormatException");
+        } catch (NumberFormatException nfe) {
+            verifyException(nfe, "For input string: \"9223372036854775808\"");
+        }
+        try {
+            // Long.MIN_VALUE - 1
+            NumberInput.parseLong("-9223372036854775809");
+            fail("expected NumberFormatException");
+        } catch (NumberFormatException nfe) {
+            verifyException(nfe, "For input string: \"-9223372036854775809\"");
+        }
+    }
+
+    // Found by oss-fuzzer
+    @Test
+    void veryLongIntRootValue() throws Exception
+    {
+        // For some reason running multiple will tend to hide the issue;
+        // possibly due to re-use of some buffers
+        _testVeryLongIntRootValue(newStreamFactory(), MODE_DATA_INPUT);
+    }
+
+    private void _testVeryLongIntRootValue(JsonFactory jsonF, int mode) throws Exception
+    {
+        StringBuilder sb = new StringBuilder(250);
+        sb.append("-2");
+        for (int i = 0; i < 220; ++i) {
+            sb.append('0');
+        }
+        sb.append(' '); // mostly important for DataInput
+        String DOC = sb.toString();
+
+        try (JsonParser p = createParser(jsonF, mode, DOC)) {
+            assertToken(p.nextToken(), JsonToken.VALUE_NUMBER_INT);
+            assertNotNull(p.getBigIntegerValue());
+        }
+    }
+
     /*
     /**********************************************************************
     /* Tests, Long
     /**********************************************************************
      */
 
-    public void testSimpleLong() throws Exception
+    @Test
+    void simpleLong() throws Exception
     {
         _testSimpleLong(MODE_INPUT_STREAM);
         _testSimpleLong(MODE_INPUT_STREAM_THROTTLED);
@@ -225,8 +302,8 @@ public class NumberParsingTest
     private void _testSimpleLong(int mode) throws Exception
     {
         long EXP_L = 12345678907L;
-        
-        JsonParser p = createParser(mode, "[ "+EXP_L+" ]");
+
+        JsonParser p = createParser(jsonFactory(), mode, "[ "+EXP_L+" ]");
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         // beyond int, should be long
@@ -234,7 +311,7 @@ public class NumberParsingTest
         assertEquals(""+EXP_L, p.getText());
 
         assertEquals(EXP_L, p.getLongValue());
-        // Should get an exception if trying to convert to int 
+        // Should get an exception if trying to convert to int
         try {
             p.getIntValue();
         } catch (InputCoercionException e) {
@@ -243,22 +320,25 @@ public class NumberParsingTest
             assertEquals(Integer.TYPE, e.getTargetType());
         }
         assertEquals((double) EXP_L, p.getDoubleValue());
-        assertEquals(BigDecimal.valueOf((long) EXP_L), p.getDecimalValue());
+        assertEquals(BigDecimal.valueOf(EXP_L), p.getDecimalValue());
         p.close();
     }
 
-    public void testLongRange() throws Exception
+    @Test
+    void longRange() throws Exception
     {
         for (int mode : ALL_MODES) {
             long belowMinInt = -1L + Integer.MIN_VALUE;
             long aboveMaxInt = 1L + Integer.MAX_VALUE;
-            String input = "[ "+Long.MAX_VALUE+","+Long.MIN_VALUE+","+aboveMaxInt+", "+belowMinInt+" ]";
-            JsonParser p = createParser(mode, input);
+            long belowMaxLong = -1L + Long.MAX_VALUE;
+            long aboveMinLong = 1L + Long.MIN_VALUE;
+            String input = "[ "+Long.MAX_VALUE+","+Long.MIN_VALUE+","+aboveMaxInt+", "+belowMinInt+","+belowMaxLong+", "+aboveMinLong+" ]";
+            JsonParser p = createParser(jsonFactory(), mode, input);
             assertToken(JsonToken.START_ARRAY, p.nextToken());
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.LONG, p.getNumberType());
             assertEquals(Long.MAX_VALUE, p.getLongValue());
-        
+
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.LONG, p.getNumberType());
             assertEquals(Long.MIN_VALUE, p.getLongValue());
@@ -271,20 +351,30 @@ public class NumberParsingTest
             assertEquals(JsonParser.NumberType.LONG, p.getNumberType());
             assertEquals(belowMinInt, p.getLongValue());
 
-            
-            assertToken(JsonToken.END_ARRAY, p.nextToken());        
+            assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+            assertEquals(JsonParser.NumberType.LONG, p.getNumberType());
+            assertEquals(belowMaxLong, p.getLongValue());
+
+            assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+            assertEquals(JsonParser.NumberType.LONG, p.getNumberType());
+            assertEquals(aboveMinLong, p.getLongValue());
+
+
+            assertToken(JsonToken.END_ARRAY, p.nextToken());
             p.close();
         }
     }
 
-    public void testLongParsing() throws Exception
+    @Test
+    void longParsing() throws Exception
     {
         char[] testChars = "123456789012345678".toCharArray();
 
         assertEquals(123456789012345678L, NumberInput.parseLong(testChars, 0, testChars.length));
     }
 
-    public void testLongBoundsChecks() throws Exception
+    @Test
+    void longBoundsChecks() throws Exception
     {
         String minLong = String.valueOf(Long.MIN_VALUE).substring(1);
         String belowMinLong = BigInteger.valueOf(Long.MIN_VALUE).subtract(BigInteger.ONE)
@@ -319,14 +409,15 @@ public class NumberParsingTest
         cbuf = aboveMaxLong.toCharArray();
         assertFalse(NumberInput.inLongRange(cbuf, 0, cbuf.length, false));
     }
-    
+
     /*
     /**********************************************************************
     /* Tests, BigXxx
     /**********************************************************************
      */
-    
-    public void testBigDecimalRange() throws Exception
+
+    @Test
+    void bigDecimalRange() throws Exception
     {
         for (int mode : ALL_MODES) {
             // let's test first values outside of Long range
@@ -335,7 +426,7 @@ public class NumberParsingTest
             BigInteger big = new BigDecimal(Long.MAX_VALUE).toBigInteger();
             big = big.add(BigInteger.ONE);
             String input = "[ "+small+"  ,  "+big+"]";
-            JsonParser p = createParser(mode, input);
+            JsonParser p = createParser(jsonFactory(), mode, input);
             assertToken(JsonToken.START_ARRAY, p.nextToken());
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
@@ -343,13 +434,14 @@ public class NumberParsingTest
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
             assertEquals(big, p.getBigIntegerValue());
-            assertToken(JsonToken.END_ARRAY, p.nextToken());        
+            assertToken(JsonToken.END_ARRAY, p.nextToken());
             p.close();
         }
     }
 
     // for [core#78]
-    public void testBigNumbers() throws Exception
+    @Test
+    void bigNumbers() throws Exception
     {
         StringBuilder sb = new StringBuilder();
         for (int i = 0; i < 520; ++i) { // input buffer is 512 bytes by default
@@ -359,12 +451,92 @@ public class NumberParsingTest
         BigInteger biggie = new BigInteger(NUMBER_STR);
 
         for (int mode : ALL_MODES) {
-            JsonParser p = createParser(mode, NUMBER_STR +" ");
-            assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
-            assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
-            assertEquals(NUMBER_STR, p.getText());
-            assertEquals(biggie, p.getBigIntegerValue());
-            p.close();
+            try (JsonParser p = createParser(jsonFactory(), mode, NUMBER_STR +" ")) {
+                assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+                assertEquals(JsonParser.NumberType.BIG_INTEGER, p.getNumberType());
+                assertEquals(NUMBER_STR, p.getText());
+                assertEquals(biggie, p.getBigIntegerValue());
+            }
+        }
+    }
+
+    // Related to [core#1135]: JsonParser.isNaN() should not be fooled
+    // by possible Double overflow
+    @Test
+    void biggerThanFloatHandling() throws Exception
+    {
+        BigDecimal tooBig = BigDecimal.valueOf(Double.MAX_VALUE);
+        tooBig = tooBig.add(tooBig);
+        // Otherwise becomes an int, need to add fractional part
+        tooBig = tooBig.add(BigDecimal.valueOf(0.25));
+        String tooBigString = tooBig.toPlainString();
+
+        // sanity check that it is beyond Double range:
+        assertTrue(new Double(tooBigString).isInfinite());
+
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(jsonFactory(), mode, tooBigString +" ")) {
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                // NOTE! We MUST NOT call `p.getNumberType()` as that would make
+                // it `NumberType.DOUBLE`
+//                assertEquals(JsonParser.NumberType.BIG_DECIMAL, p.getNumberType());
+                assertFalse(p.isNaN());
+                assertEquals(tooBigString, p.getText());
+                assertEquals(tooBig, p.getDecimalValue());
+                assertFalse(p.isNaN());
+            }
+        }
+    }
+
+    /*
+    /**********************************************************************
+    /* Tests, int/long/BigInteger via E-notation (engineering)
+    /**********************************************************************
+     */
+    
+    @Test
+    void bigIntegerWithENotation() throws Exception {
+        final String DOC = "1e5 ";
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(jsonFactory(), mode, DOC)) {
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(100000L, p.getBigIntegerValue().longValue());
+            }
+        }
+    }
+
+    @Test
+    void longWithENotation() throws Exception {
+        final String DOC = "1e5 ";
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(jsonFactory(), mode, DOC)) {
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(100000L, p.getLongValue());
+            }
+        }
+    }
+
+    @Test
+    void intWithENotation() throws Exception {
+        final String DOC = "1e5 ";
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(jsonFactory(), mode, DOC)) {
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(100000, p.getIntValue());
+            }
+        }
+    }
+
+    @Test
+    void largeBigIntegerWithENotation() throws Exception {
+        // slightly bigger than Double.MAX_VALUE (we need to avoid parsing as double in this case)
+        final String DOC = "2e308 ";
+        final BigInteger expected = new BigDecimal(DOC.trim()).toBigInteger();
+        for (int mode : ALL_MODES) {
+            try (JsonParser p = createParser(jsonFactory(), mode, DOC)) {
+                assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
+                assertEquals(expected, p.getBigIntegerValue());
+            }
         }
     }
 
@@ -373,11 +545,12 @@ public class NumberParsingTest
     /* Tests, floating point (basic)
     /**********************************************************************
      */
-    
-    public void testSimpleDouble() throws Exception
+
+    @Test
+    void simpleDouble() throws Exception
     {
         final String[] INPUTS = new String[] {
-            "1234.00", "2.1101567E-16", "1.0e5", "0.0", "1.0", "-1.0", 
+            "1234.00", "2.1101567E-16", "1.0e5", "0.0", "1.0", "-1.0",
             "-0.5", "-12.9", "-999.0",
             "2.5e+5", "9e4", "-12e-3", "0.25",
         };
@@ -385,12 +558,12 @@ public class NumberParsingTest
             for (int i = 0; i < INPUTS.length; ++i) {
 
                 // First in array
-                
+
                 String STR = INPUTS[i];
                 double EXP_D = Double.parseDouble(STR);
                 String DOC = "["+STR+"]";
 
-                JsonParser p = createParser(mode, DOC+" ");
+                JsonParser p = createParser(jsonFactory(), mode, DOC+" ");
                 assertToken(JsonToken.START_ARRAY, p.nextToken());
 
                 assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
@@ -403,7 +576,7 @@ public class NumberParsingTest
                 p.close();
 
                 // then outside
-                p = createParser(mode, STR + " ");
+                p = createParser(jsonFactory(), mode, STR + " ");
                 JsonToken t = null;
 
                 try {
@@ -411,7 +584,7 @@ public class NumberParsingTest
                 } catch (Exception e) {
                     throw new Exception("Failed to parse input '"+STR+"' (parser of type "+p.getClass().getSimpleName()+")", e);
                 }
-                
+
                 assertToken(JsonToken.VALUE_NUMBER_FLOAT, t);
                 assertEquals(STR, p.getText());
                 if (mode != MODE_DATA_INPUT) {
@@ -422,7 +595,8 @@ public class NumberParsingTest
         }
     }
 
-    public void testFloatBoundary146Chars() throws Exception
+    @Test
+    void floatBoundary146Chars() throws Exception
     {
         final char[] arr = new char[50005];
         for(int i = 500; i != 9000; ++i) {
@@ -433,13 +607,14 @@ public class NumberParsingTest
           arr[i + 3] = '-';
           arr[i + 4] = '1';
           CharArrayReader r = new CharArrayReader(arr, 0, i+5);
-          JsonParser p = FACTORY.createParser(r);
+          JsonParser p = jsonFactory().createParser(r);
           assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
           p.close();
         }
     }
 
-    public void testFloatBoundary146Bytes() throws Exception
+    @Test
+    void floatBoundary146Bytes() throws Exception
     {
         final byte[] arr = new byte[50005];
         for(int i = 500; i != 9000; ++i) {
@@ -450,107 +625,41 @@ public class NumberParsingTest
             arr[i + 3] = '-';
             arr[i + 4] = '1';
             ByteArrayInputStream in = new ByteArrayInputStream(arr, 0, i+5);
-            JsonParser p = FACTORY.createParser(in);
+            JsonParser p = jsonFactory().createParser(in);
             assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
             p.close();
         }
     }
 
-    /*
-    /**********************************************************************
-    /* Tests, BigDecimal (core#677)
-    /**********************************************************************
-     */
-
-    public void testBigBigDecimalsBytes() throws Exception
-    {
-        _testBigBigDecimals(MODE_INPUT_STREAM);
-        _testBigBigDecimals(MODE_INPUT_STREAM_THROTTLED);
-    }
-
-    public void testBigBigDecimalsChars() throws Exception
-    {
-        _testBigBigDecimals(MODE_READER);
-    }
-
-    public void testBigBigDecimalsDataInput() throws Exception
-    {
-        _testBigBigDecimals(MODE_DATA_INPUT);
-    }
-
-    private void _testBigBigDecimals(int mode) throws Exception
-    {
-        final String BASE_FRACTION =
- "01610253934481930774151441507943554511027782188707463024288149352877602369090537"
-+"80583522838238149455840874862907649203136651528841378405339370751798532555965157588"
-+"51877960056849468879933122908090021571162427934915567330612627267701300492535817858"
-+"36107216979078343419634586362681098115326893982589327952357032253344676618872460059"
-+"52652865429180458503533715200184512956356092484787210672008123556320998027133021328"
-+"04777044107393832707173313768807959788098545050700242134577863569636367439867566923"
-+"33479277494056927358573496400831024501058434838492057410330673302052539013639792877"
-+"76670882022964335417061758860066263335250076803973514053909274208258510365484745192"
-+"39425298649420795296781692303253055152441850691276044546565109657012938963181532017"
-+"97420631515930595954388119123373317973532146157980827838377034575940814574561703270"
-+"54949003909864767732479812702835339599792873405133989441135669998398892907338968744"
-+"39682249327621463735375868408190435590094166575473967368412983975580104741004390308"
-+"45302302121462601506802738854576700366634229106405188353120298347642313881766673834"
-+"60332729485083952142460470270121052469394888775064758246516888122459628160867190501"
-+"92476878886543996441778751825677213412487177484703116405390741627076678284295993334"
-+"23142914551517616580884277651528729927553693274406612634848943914370188078452131231"
-+"17351787166509190240927234853143290940647041705485514683182501795615082930770566118"
-+"77488417962195965319219352314664764649802231780262169742484818333055713291103286608"
-+"64318433253572997833038335632174050981747563310524775762280529871176578487487324067"
-+"90242862159403953039896125568657481354509805409457993946220531587293505986329150608"
-+"18702520420240989908678141379300904169936776618861221839938283876222332124814830207"
-+"073816864076428273177778788053613345444299361357958409716099682468768353446625063";
-
-        for (String asText : new String[] {
-                "50."+BASE_FRACTION,
-                "-37."+BASE_FRACTION,
-                "0.00"+BASE_FRACTION,
-                "-0.012"+BASE_FRACTION,
-                "9999998."+BASE_FRACTION,
-                "-8888392."+BASE_FRACTION,
-        }) {
-            final String DOC = "[ "+asText+" ]";
-
-            JsonParser p = createParser(mode, DOC);
-            assertToken(JsonToken.START_ARRAY, p.nextToken());
-            assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
-            final BigDecimal exp = new BigDecimal(asText);
-            assertEquals(exp, p.getDecimalValue());
-            p.close();
-        }
-    }
-
     /*
     /**********************************************************************
     /* Tests, misc other
     /**********************************************************************
      */
 
-    public void testNumbers() throws Exception
+    @Test
+    void numbers() throws Exception
     {
         _testNumbers(MODE_INPUT_STREAM);
         _testNumbers(MODE_INPUT_STREAM_THROTTLED);
         _testNumbers(MODE_READER);
         _testNumbers(MODE_DATA_INPUT);
     }
-    
+
     private void _testNumbers(int mode) throws Exception
     {
         final String DOC = "[ -13, 8100200300, 13.5, 0.00010, -2.033 ]";
 
-        JsonParser p = createParser(mode, DOC);
+        JsonParser p = createParser(jsonFactory(), mode, DOC);
 
         assertToken(JsonToken.START_ARRAY, p.nextToken());
-        
+
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(-13, p.getIntValue());
         assertEquals(-13L, p.getLongValue());
         assertEquals(-13., p.getDoubleValue());
         assertEquals("-13", p.getText());
-        
+
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(8100200300L, p.getLongValue());
         // Should get exception for overflow:
@@ -562,7 +671,7 @@ public class NumberParsingTest
         }
         assertEquals(8100200300.0, p.getDoubleValue());
         assertEquals("8100200300", p.getText());
-        
+
         assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
         assertEquals(13, p.getIntValue());
         assertEquals(13L, p.getLongValue());
@@ -574,7 +683,7 @@ public class NumberParsingTest
         assertEquals(0L, p.getLongValue());
         assertEquals(0.00010, p.getDoubleValue());
         assertEquals("0.00010", p.getText());
-        
+
         assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
         assertEquals(-2, p.getIntValue());
         assertEquals(-2L, p.getLongValue());
@@ -586,12 +695,13 @@ public class NumberParsingTest
         p.close();
     }
 
-    
+
     /**
      * Method that tries to test that number parsing works in cases where
      * input is split between buffer boundaries.
      */
-    public void testParsingOfLongerSequences() throws Exception
+    @Test
+    void parsingOfLongerSequences() throws Exception
     {
         double[] values = new double[] { 0.01, -10.5, 2.1e9, 4.0e-8 };
         StringBuilder sb = new StringBuilder();
@@ -629,9 +739,9 @@ public class NumberParsingTest
             JsonParser p;
 
             if (input == 0) {
-                p = createParserUsingStream(DOC, "UTF-8");
+                p = createParserUsingStream(jsonFactory(), DOC, "UTF-8");
             } else {
-                p = FACTORY.createParser(DOC);
+                p = jsonFactory().createParser(DOC);
             }
 
             assertToken(JsonToken.START_ARRAY, p.nextToken());
@@ -647,10 +757,12 @@ public class NumberParsingTest
     }
 
     // [jackson-core#157]
-    public void testLongNumbers() throws Exception
+    // 19-Dec-2022, tatu: Reduce length so as not to hit too-long-number limit
+    @Test
+    void longNumbers() throws Exception
     {
-        StringBuilder sb = new StringBuilder(9000);
-        for (int i = 0; i < 9000; ++i) {
+        StringBuilder sb = new StringBuilder(900);
+        for (int i = 0; i < 900; ++i) {
             sb.append('9');
         }
         String NUM = sb.toString();
@@ -659,21 +771,23 @@ public class NumberParsingTest
         _testLongNumbers(f, NUM, false);
         _testLongNumbers(f, NUM, true);
     }
-    
+
     private void _testLongNumbers(JsonFactory f, String num, boolean useStream) throws Exception
     {
         final String doc = "[ "+num+" ]";
-        JsonParser p = useStream
+        try (JsonParser p = useStream
                 ? f.createParser(doc.getBytes("UTF-8"))
-                        : f.createParser(doc);
-        assertToken(JsonToken.START_ARRAY, p.nextToken());
-        assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
-        assertEquals(num, p.getText());
-        assertToken(JsonToken.END_ARRAY, p.nextToken());
+                        : f.createParser(doc)) {
+            assertToken(JsonToken.START_ARRAY, p.nextToken());
+            assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+            assertEquals(num, p.getText());
+            assertToken(JsonToken.END_ARRAY, p.nextToken());
+        }
     }
 
     // and alternate take on for #157 (with negative num)
-    public void testLongNumbers2() throws Exception
+    @Test
+    void longNumbers2() throws Exception
     {
         StringBuilder input = new StringBuilder();
         // test this with negative
@@ -682,7 +796,9 @@ public class NumberParsingTest
             input.append(1);
         }
         final String DOC = input.toString();
-        JsonFactory f = new JsonFactory();
+        JsonFactory f = JsonFactory.builder()
+                .streamReadConstraints(StreamReadConstraints.builder().maxNumberLength(10000).build())
+                .build();
         _testIssue160LongNumbers(f, DOC, false);
         _testIssue160LongNumbers(f, DOC, true);
     }
@@ -690,8 +806,8 @@ public class NumberParsingTest
     private void _testIssue160LongNumbers(JsonFactory f, String doc, boolean useStream) throws Exception
     {
         JsonParser p = useStream
-                ? FACTORY.createParser(doc.getBytes("UTF-8"))
-                        : FACTORY.createParser(doc);
+                ? f.createParser(doc.getBytes("UTF-8"))
+                : f.createParser(doc);
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         BigInteger v = p.getBigIntegerValue();
         assertNull(p.nextToken());
@@ -703,7 +819,8 @@ public class NumberParsingTest
      * Method that tries to test that number parsing works in cases where
      * input is split between buffer boundaries.
      */
-    public void testParsingOfLongerSequencesWithNonNumeric() throws Exception
+    @Test
+    void parsingOfLongerSequencesWithNonNumeric() throws Exception
     {
         JsonFactory f = JsonFactory.builder()
                 .enable(JsonReadFeature.ALLOW_NON_NUMERIC_NUMBERS)
@@ -759,7 +876,8 @@ public class NumberParsingTest
     /**********************************************************
      */
 
-    public void testInvalidBooleanAccess() throws Exception {
+    @Test
+    void invalidBooleanAccess() throws Exception {
         _testInvalidBooleanAccess(MODE_INPUT_STREAM);
         _testInvalidBooleanAccess(MODE_INPUT_STREAM_THROTTLED);
         _testInvalidBooleanAccess(MODE_READER);
@@ -768,7 +886,7 @@ public class NumberParsingTest
 
     private void _testInvalidBooleanAccess(int mode) throws Exception
     {
-        JsonParser p = createParser(mode, "[ \"abc\" ]");
+        JsonParser p = createParser(jsonFactory(), mode, "[ \"abc\" ]");
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         try {
@@ -780,16 +898,17 @@ public class NumberParsingTest
         p.close();
     }
 
-    public void testInvalidIntAccess() throws Exception {
+    @Test
+    void invalidIntAccess() throws Exception {
         _testInvalidIntAccess(MODE_INPUT_STREAM);
         _testInvalidIntAccess(MODE_INPUT_STREAM_THROTTLED);
         _testInvalidIntAccess(MODE_READER);
         _testInvalidIntAccess(MODE_DATA_INPUT);
     }
-    
+
     private void _testInvalidIntAccess(int mode) throws Exception
     {
-        JsonParser p = createParser(mode, "[ \"abc\" ]");
+        JsonParser p = createParser(jsonFactory(), mode, "[ \"abc\" ]");
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         try {
@@ -801,16 +920,17 @@ public class NumberParsingTest
         p.close();
     }
 
-    public void testInvalidLongAccess() throws Exception {
+    @Test
+    void invalidLongAccess() throws Exception {
         _testInvalidLongAccess(MODE_INPUT_STREAM);
         _testInvalidLongAccess(MODE_INPUT_STREAM_THROTTLED);
         _testInvalidLongAccess(MODE_READER);
         _testInvalidLongAccess(MODE_DATA_INPUT);
     }
-    
+
     private void _testInvalidLongAccess(int mode) throws Exception
     {
-        JsonParser p = createParser(mode, "[ false ]");
+        JsonParser p = createParser(jsonFactory(), mode, "[ false ]");
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_FALSE, p.nextToken());
         try {
@@ -823,7 +943,8 @@ public class NumberParsingTest
     }
 
     // [core#317]
-    public void testLongerFloatingPoint() throws Exception
+    @Test
+    void longerFloatingPoint() throws Exception
     {
         StringBuilder input = new StringBuilder();
         for (int i = 1; i < 201; i++) {
@@ -835,11 +956,11 @@ public class NumberParsingTest
         // test out with both Reader and ByteArrayInputStream
         JsonParser p;
 
-        p = FACTORY.createParser(new StringReader(DOC));
+        p = jsonFactory().createParser(new StringReader(DOC));
         _testLongerFloat(p, DOC);
         p.close();
-        
-        p = FACTORY.createParser(new ByteArrayInputStream(DOC.getBytes("UTF-8")));
+
+        p = jsonFactory().createParser(new ByteArrayInputStream(DOC.getBytes("UTF-8")));
         _testLongerFloat(p, DOC);
         p.close();
     }
@@ -851,9 +972,10 @@ public class NumberParsingTest
         assertNull(p.nextToken());
     }
 
-    public void testInvalidNumber() throws Exception {
+    @Test
+    void invalidNumber() throws Exception {
         for (int mode : ALL_MODES) {
-            JsonParser p = createParser(mode, " -foo ");
+            JsonParser p = createParser(jsonFactory(), mode, " -foo ");
             try {
                 p.nextToken();
                 fail("Should not pass");
@@ -864,6 +986,16 @@ public class NumberParsingTest
         }
     }
 
+    @Test
+    void negativeMaxNumberLength() {
+        try {
+            StreamReadConstraints src = StreamReadConstraints.builder().maxNumberLength(-1).build();
+            fail("expected IllegalArgumentException; instead built: "+src);
+        } catch (IllegalArgumentException iae) {
+            verifyException(iae, "Cannot set maxNumberLength to a negative value");
+        }
+    }
+
     /*
     /**********************************************************
     /* Helper methods
