@@ -5,33 +5,45 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.io.SerializedString;
 
 import java.io.*;
+
+import static org.junit.jupiter.api.Assertions.*;
+
 import java.util.Random;
 
+import org.junit.jupiter.api.Test;
+
 /**
  * Set of basic unit tests for verifying that the basic parser
  * functionality works as expected.
  */
 public class UTF8NamesParseTest
-    extends BaseTest
+    extends JUnit5TestBase
 {
     final static String[] UTF8_2BYTE_STRINGS = new String[] {
-        /* This may look funny, but UTF8 scanner has fairly
-         * elaborate decoding machinery, and it is indeed
-         * necessary to try out various combinations...
-         */
-        "b", "A\u00D8", "abc", "c3p0",
-        "12345", "......", "Long\u00FAer",
+        // This may look funny, but UTF8 scanner has fairly elaborate decoding
+        // machinery, and it is indeed necessary to try out various combinations...
+        "b", "\u00d8", "A\u00D8", "ab\u00D8d", "abc\u00d8", "c3p0",
+        "1234\u00C75",
+        "......",
+        "Long\u00FAer",
         "Latin1-fully-\u00BE-develop\u00A8d",
         "Some very long name, ridiculously long actually to see that buffer expansion works: \u00BF?"
     };
 
     final static String[] UTF8_3BYTE_STRINGS = new String[] {
-        "\uC823?", "A\u400F", "1\u1234?",
+        "\uC823?", "A\u400F", "1\u1234?", "ab\u1234d",
         "Ab123\u4034",
-        "Even-longer:\uC023"
+        "Long \uC023 ish",
+        "Bit longer:\uC023",
+        "Even-longer:\u3456",
+        "Yet bit longer \uC023",
+        "Even more \u3456 longer",
+        "\uC023 Possibly ridiculous",
+        "But \uC023 this takes the cake",
     };
 
-    public void testEmptyName() throws Exception
+    @Test
+    void emptyName() throws Exception
     {
         _testEmptyName(MODE_INPUT_STREAM);
         _testEmptyName(MODE_INPUT_STREAM_THROTTLED);
@@ -44,14 +56,15 @@ public class UTF8NamesParseTest
         JsonParser p = createParser(mode, DOC);
         assertToken(JsonToken.START_OBJECT, p.nextToken());
         assertToken(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("", p.getCurrentName());
+        assertEquals("", p.currentName());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         assertEquals("", p.getText());
         assertToken(JsonToken.END_OBJECT, p.nextToken());
         p.close();
     }
 
-    public void testUtf8Name2Bytes() throws Exception
+    @Test
+    void utf8Name2Bytes() throws Exception
     {
         _testUtf8Name2Bytes(MODE_INPUT_STREAM);
         _testUtf8Name2Bytes(MODE_INPUT_STREAM_THROTTLED);
@@ -72,21 +85,22 @@ public class UTF8NamesParseTest
 
             assertTrue(p.hasToken(JsonToken.FIELD_NAME));
             assertTrue(p.hasTokenId(JsonTokenId.ID_FIELD_NAME));
-            
-            assertEquals(NAME, p.getCurrentName());
+
+            assertEquals(NAME, p.currentName());
             assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
             assertTrue(p.hasToken(JsonToken.VALUE_NUMBER_INT));
             assertTrue(p.hasTokenId(JsonTokenId.ID_NUMBER_INT));
 
             // should retain name during value entry, too
-            assertEquals(NAME, p.getCurrentName());
-            
+            assertEquals(NAME, p.currentName());
+
             assertToken(JsonToken.END_OBJECT, p.nextToken());
             p.close();
         }
     }
 
-    public void testUtf8Name3Bytes() throws Exception
+    @Test
+    void utf8Name3Bytes() throws Exception
     {
         _testUtf8Name3Bytes(MODE_INPUT_STREAM);
         _testUtf8Name3Bytes(MODE_DATA_INPUT);
@@ -103,27 +117,28 @@ public class UTF8NamesParseTest
 
             JsonParser p = createParser(mode, DOC);
             assertToken(JsonToken.START_OBJECT, p.nextToken());
-            
+
             assertToken(JsonToken.FIELD_NAME, p.nextToken());
-            assertEquals(NAME, p.getCurrentName());
+            assertEquals(NAME, p.currentName());
             assertToken(JsonToken.VALUE_TRUE, p.nextToken());
-            assertEquals(NAME, p.getCurrentName());
-            
+            assertEquals(NAME, p.currentName());
+
             assertToken(JsonToken.END_OBJECT, p.nextToken());
-            
+
             p.close();
         }
     }
 
     // How about tests for Surrogate-Pairs?
 
-    public void testUtf8StringTrivial() throws Exception
+    @Test
+    void utf8StringTrivial() throws Exception
     {
         _testUtf8StringTrivial(MODE_INPUT_STREAM);
         _testUtf8StringTrivial(MODE_DATA_INPUT);
         _testUtf8StringTrivial(MODE_INPUT_STREAM_THROTTLED);
     }
-    
+
     public void _testUtf8StringTrivial(int mode) throws Exception
     {
         String[] VALUES = UTF8_2BYTE_STRINGS;
@@ -155,7 +170,8 @@ public class UTF8NamesParseTest
         }
     }
 
-    public void testUtf8StringValue() throws Exception
+    @Test
+    void utf8StringValue() throws Exception
     {
         _testUtf8StringValue(MODE_INPUT_STREAM, 2900);
         _testUtf8StringValue(MODE_DATA_INPUT, 2900);
@@ -190,17 +206,15 @@ public class UTF8NamesParseTest
             sb.append((char) c);
         }
 
+        final String VALUE = sb.toString();
         ByteArrayOutputStream bout = new ByteArrayOutputStream(len + (len >> 2));
         OutputStreamWriter out = new OutputStreamWriter(bout, "UTF-8");
         out.write("[\"");
-        String VALUE = sb.toString();
         out.write(VALUE);
         out.write("\"]");
         out.close();
 
-        byte[] data = bout.toByteArray();
-
-        JsonParser p = createParser(mode, data);
+        JsonParser p = createParser(mode, bout.toByteArray());
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         String act = p.getText();
@@ -208,9 +222,32 @@ public class UTF8NamesParseTest
         assertEquals(VALUE.length(), act.length());
         assertEquals(VALUE, act);
         p.close();
+
+        // But how about as key
+        bout = new ByteArrayOutputStream(len + (len >> 2));
+        out = new OutputStreamWriter(bout, "UTF-8");
+        out.write("{\"");
+        out.write(VALUE);
+        out.write("\":42}");
+        out.close();
+
+        p = createParser(mode, bout.toByteArray());
+        assertToken(JsonToken.START_OBJECT, p.nextToken());
+        assertToken(JsonToken.FIELD_NAME, p.nextToken());
+
+        act = p.getText();
+        assertEquals(VALUE.length(), act.length());
+        assertEquals(VALUE, act);
+
+        assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
+        assertEquals(42, p.getIntValue());
+        assertToken(JsonToken.END_OBJECT, p.nextToken());
+
+        p.close();
     }
 
-    public void testNextFieldName() throws IOException
+    @Test
+    void nextFieldName() throws IOException
     {
 		ByteArrayOutputStream os = new ByteArrayOutputStream();
 		os.write('{');
@@ -230,10 +267,10 @@ public class UTF8NamesParseTest
     {
         SerializedString id = new SerializedString("id");
         JsonParser parser = createParser(mode, doc);
-        assertEquals(parser.nextToken(), JsonToken.START_OBJECT);
+        assertEquals(JsonToken.START_OBJECT, parser.nextToken());
         assertTrue(parser.nextFieldName(id));
-        assertEquals(parser.nextToken(), JsonToken.VALUE_NUMBER_INT);
-        assertEquals(parser.nextToken(), JsonToken.END_OBJECT);
+        assertEquals(JsonToken.VALUE_NUMBER_INT, parser.nextToken());
+        assertEquals(JsonToken.END_OBJECT, parser.nextToken());
         parser.close();
     }
 }
