@@ -1,36 +1,45 @@
 package com.fasterxml.jackson.core.util;
 
+import org.junit.jupiter.api.Test;
+
 import com.fasterxml.jackson.core.Version;
 import com.fasterxml.jackson.core.json.PackageVersion;
 import com.fasterxml.jackson.core.json.UTF8JsonGenerator;
 
-public class TestVersionUtil extends com.fasterxml.jackson.core.BaseTest
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
+class TestVersionUtil extends com.fasterxml.jackson.core.JUnit5TestBase
 {
-    public void testVersionPartParsing()
+    @Test
+    void versionPartParsing()
     {
         assertEquals(13, VersionUtil.parseVersionPart("13"));
         assertEquals(27, VersionUtil.parseVersionPart("27.8"));
         assertEquals(0, VersionUtil.parseVersionPart("-3"));
     }
 
-    public void testVersionParsing()
+    @Test
+    void versionParsing()
     {
         assertEquals(new Version(1, 2, 15, "foo", "group", "artifact"),
                 VersionUtil.parseVersion("1.2.15-foo", "group", "artifact"));
     }
 
     @SuppressWarnings("deprecation")
-    public void testMavenVersionParsing() {
+    @Test
+    void mavenVersionParsing() {
         assertEquals(new Version(1, 2, 3, "SNAPSHOT", "foo.bar", "foo-bar"),
                 VersionUtil.mavenVersionFor(TestVersionUtil.class.getClassLoader(), "foo.bar", "foo-bar"));
     }
 
-    public void testPackageVersionMatches() {
+    @Test
+    void packageVersionMatches() {
         assertEquals(PackageVersion.VERSION, VersionUtil.versionFor(UTF8JsonGenerator.class));
     }
 
     // [core#248]: make sure not to return `null` but `Version.unknownVersion()`
-    public void testVersionForUnknownVersion() {
+    @Test
+    void versionForUnknownVersion() {
         // expecting return version.unknownVersion() instead of null
         assertEquals(Version.unknownVersion(), VersionUtil.versionFor(TestVersionUtil.class));
     }
